------------------------------------------------------------------------------
-- BUSH Built-in Shell Commands                                             --
--                                                                          --
-- Part of BUSH                                                             --
------------------------------------------------------------------------------
--                                                                          --
--                 Copyright (C) 2001 Ken O. Burtch & FSF                   --
--                                                                          --
-- This is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion.  This is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with this;  see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- This is maintained at http://www.vaxxine.com/pegasoft                    --
--                                                                          --
------------------------------------------------------------------------------

with bush_os;
use  bush_os;

with ada.text_io,ada.strings.unbounded.text_io,ada.strings.fixed;
use  ada.text_io,ada.strings.unbounded.text_io,ada.strings.fixed;

package body builtins is

-- PWD
-- loosely modelled on bash/builtins/cd.def

procedure findPwd is
-- lookup current working directory and assign it to our
-- current_working_directory string, else assign it a null string
  buffer : string( 1..4096 );
begin
  errno := 0;
  getcwd( buffer, buffer'length );
  if errno /= 0 then
     put_line( standard_error, "findPwd: error getting current working directory, errno "
	& errno'img );
     current_working_directory := null_unbounded_string;
  end if; 
  current_working_directory := to_unbounded_string(
     buffer( 1..index( buffer, ASCII.NUL & "" ) - 1 ) ) ;
end findPwd;

procedure bindPwd( symlinks : boolean := false ) is
begin
  findPwd;
  if current_working_directory /= null_unbounded_string then
     -- SYMBOLIC LINKS TO BE HANDLED HERE LATER
     --findIdent( to_unbounded_string( "PWD" ), pwd );
     --findIdent( to_unbounded_string( "OLDPWD" ), oldpwd );
     --if identifiers( pwd ).kind = string_t then
     --   if identifiers( oldpwd ).kind = string_t then
     --      identifiers( oldpwd ).value := identifiers( pwd ).value;
     --   end if;
     --   identifiers( pwd ).value := current_working_directory;
     --end if;
     null;
  end if;
end bindPwd;

function pwd return unbounded_string is
-- pwd: present working directory
begin
   bindPwd;
   return current_working_directory;
end pwd;

-- CD
-- loosely modelled on bash/builtins/cd.def

procedure cd( s : unbounded_string ) is
-- cd: change directory
  pwd, oldpwd : identifier;
  path : unbounded_string := s;
  showPath : boolean := false;
  temp_id : identifier;
begin
  if length( path ) = 0 then                 -- no path?
     path := to_unbounded_string( "$HOME" ); -- then go home
  elsif Element( path, 1 ) = '~' then        -- leading tilda?
     Delete( path, 1, 1 );                   -- shortform for
     Insert( path, 1, "$HOME" );             -- home
  elsif path = "-" then                      -- minus path?
     path := to_unbounded_string( "$OLDPWD" ); -- short for OLDPWD
     showPath := true;
  end if;

  if head( path, 5 ) = "$HOME" then
     findIdent( to_unbounded_string( "HOME" ), temp_id );
     Delete( path, 1, 5 );
     Insert( path, 1, to_string( identifiers( temp_id ).value ) );
  elsif head( path, 7 ) = "$OLDPWD" then
     findIdent( to_unbounded_string( "OLDPWD" ), temp_id );
     Delete( path, 1, 7 );
     Insert( path, 1, to_string( identifiers( temp_id ).value ) );
  end if;

  -- CDPATH support not yet implmeneted.  Should go HERE, but
  -- is CDPATH such a good idea anyway?  Not usually!

  if chdir( to_string( path ) & ASCII.NUL ) = 0 then
     bindPwd;
-- it should also not take into account the nesting (should be global)
     findIdent( to_unbounded_string( "PWD" ), pwd );
     findIdent( to_unbounded_string( "OLDPWD" ), oldpwd );
     if pwd /= eof_t then
        if oldpwd /= eof_t then
           identifiers( oldpwd ).value := identifiers( pwd ).value;
        end if;
        identifiers( pwd ).value := current_working_directory;
     end if;
  else
     err( "No such path '" & to_string( path ) & "'" );
  end if;
  if showPath then
     put_line( current_working_directory );
  end if;
end cd;

procedure env( id : identifier := eof_t ) is
-- env - list environment to standard output
  perc : integer;
begin
  if id /= eof_t then
     Put_Identifier( id );
     return;
  end if;
  for i in 1..identifiers_top-1 loop
    if identifiers( i ).kind /= keyword_t then
       Put_Identifier( i );
    end if;
  end loop;
  perc := integer( ( ( identifiers_top - 1 ) * 100 ) / identifiers'last );
  Put( "The symbol table is" );
  Put( perc'img );
  Put( "% full with" );
  Put( identifier'image( identifiers_top-1 ) );
  Put_Line( " identifiers." );
end env;

end builtins;
