------------------------------------------------------------------------------
-- BUSH Main Program                                                        --
--                                                                          --
-- Part of BUSH                                                             --
------------------------------------------------------------------------------
--                                                                          --
--                 Copyright (C) 2001 Ken O. Burtch & FSF                   --
--                                                                          --
-- This is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion.  This is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with this;  see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- This is maintained at http://www.vaxxine.com/pegasoft                    --
--                                                                          --
------------------------------------------------------------------------------

with ada.text_io, ada.strings.unbounded.text_io, ada.command_line, bush_os, scanner, user_io, parser;
use  ada.text_io, ada.strings.unbounded.text_io, ada.command_line, bush_os, scanner, user_io, parser;

pragma Optimize( space );

procedure bush is
begin

  -- set scanner variable to reflect whether or not this is a login
  -- shell

  isLoginShell := Command_Name = "-";

  -- collect command line options
  
  if Argument_Count = 1 then
     if Argument(1) = "-h" or Argument( 1 ) = "--help" then
        Put_Line( "BUSH (Business Shell) usage" );
        Put_Line( "bush [-check|-c|--no-check|-n][--verbose|-v][--restricted|-r] [script [param1 ...] ]" );
        Set_Exit_Status( 0 );
        return;
     end if;
  end if;
  if Argument_Count > 0 then
     optionOffset := 1;
     for i in 1..Argument_Count loop
         if Argument(i) = "-c" or Argument(i) = "--check" then
            syntaxOpt := true;
         elsif Argument(i) = "-n" or Argument(i) = "--no-check" then
            nosyntaxOpt := true;
         elsif Argument(i) = "-r" or Argument(i) = "--restricted" then
            rshOpt := true;
         elsif Argument(i) = "-v" or Argument(i) = "--verbose" then
            verboseOpt := true;
         else
            if Argument(i)'length = 0 then
               Put_Line( standard_error, "empty script name" );
               Set_Exit_Status( 1 );
               return;
            elsif Argument(i)(1) = '-' then
               Put_Line( standard_error, "unknown option: " & Argument(i) );
               Set_Exit_Status( 1 );
               return;
            else
               exit;
            end if;
         end if;
         optionOffset := optionOffset + 1;
     end loop;
  end if;
  if syntaxOpt and nosyntaxOpt then
     Put_Line( standard_error, "-c and -n cannot be used together" );
     Set_Exit_Status( 1 );
     return;
  elsif (syntaxOpt or nosyntaxOpt) and optionOffset > Argument_Count then
     Put_Line( standard_error, "missing command name" );
     Set_Exit_Status( 1 );
     return;
  end if;
  if optionOffset > Argument_Count then
     optionOffset := 0;
  end if;

  -- display initialization message if (on a terminal and not
  -- running a script)
  if isatty( stdin ) or isatty( stdout ) then
     if optionOffset = 0 then
        put_line( "Initializing BUSH..." );
     end if;
     getDisplayInfo( displayInfo );
  end if;

  -- initialize

  startScanner;
  startParser;

  -- replace initialization message with copyright notice (if on
  -- a terminal and not running a script)

  if isatty( stdout ) then
     if optionOffset = 0 then
        put( term_up );                                 -- erase the message
        put( term_cleol );
        Put_Line( "BUSH " & version & " Copyright (c) 2001 PegaSoft Canada and Free Software Foundation");
        Put_Line( "Type ""help"" for help" );
        New_Line;
     end if;
  end if;

  -- run

  Parse;

  -- shutdown

  shutdownParser;
  shutdownScanner;
end bush;
