------------------------------------------------------------------------------
-- BUSH AdaScript Language Parser                                           --
--                                                                          --
-- Part of BUSH                                                             --
------------------------------------------------------------------------------
--                                                                          --
--                 Copyright (C) 2001 Ken O. Burtch & FSF                   --
--                                                                          --
-- This is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion.  This is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with this;  see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- This is maintained at http://www.vaxxine.com/pegasoft                    --
--                                                                          --
------------------------------------------------------------------------------

with ada.text_io, ada.strings.unbounded,
    bush_os, scanner;
use ada.text_io, ada.strings.unbounded,
    bush_os, scanner;

package parser is

---------------------------------------------------------
-- GLOBAL STUFF
---------------------------------------------------------

version : constant string := "0.8";


---------------------------------------------------------
-- START OF ADASCRIPT PARSER
---------------------------------------------------------

-- This list is not a complete list of parser procedures.
-- It is primarily for child packages and forward declarations.

procedure ParseNewIdentifier( id : out identifier );
procedure ParseIdentifier( id : out identifier );

procedure ParseIsOpen( b : out identifier );
procedure ParseEndOfFile( result : out unbounded_string );
procedure ParseLine( result : out unbounded_string );
procedure ParseName( result : out unbounded_string );
procedure ParseMode( result : out unbounded_string );
procedure ParseGetLine( str : out unbounded_string );

procedure ParseFactor( f : out unbounded_string; kind : out identifier );
procedure ParsePowerTermOperator( op : out unbounded_string );
procedure ParsePowerTerm( term : out unbounded_string; term_type : out identifier );
procedure ParseTermOperator( op : out unbounded_string );
procedure ParseTerm( term : out unbounded_string; term_type : out identifier );
procedure ParseSimpleExpressionOperator( op : out unbounded_string );
procedure ParseSimpleExpression( se : out unbounded_string; expr_type : out identifier );
procedure ParseRelationalOperator( op : out unbounded_string );
procedure ParseRelation( re : out unbounded_string; rel_type : out identifier );
procedure ParseExpressionOperator( op : out identifier );
procedure ParseExpression( ex : out unbounded_string; expr_type : out identifier );
procedure ParseAssignPart( expr_value : out unbounded_string; expr_type : out identifier );
procedure ParseDeclarationPart( id : identifier );
procedure ParsePragma;
procedure ParseType;
procedure ParseSubtype;
procedure ParseIfBlock;
procedure ParseWhileBlock;
procedure ParseForBlock;

procedure ParseOpenFile( f : out identifier );
procedure ParseOpenFileOrSocket( f, kind : out identifier );
procedure ParseClosedFile( f : out identifier );
procedure ParseOpen( create : boolean := false );
procedure ParseReset;
procedure ParseClose;
procedure ParseDelete;
procedure ParseSkipLine;
procedure ParsePutLine;
procedure ParseQuestion;
procedure ParsePut;
procedure ParseNewLine;
procedure ParseSetInput;
procedure ParseSetOutput;
procedure ParseSetError;

procedure ParseSystem;
procedure ParseDelay;
procedure ParseUnset;
procedure ParseTypeset;
procedure ParseEnvironment;
procedure ParseClear;
procedure ParseTrace;
procedure ParseHelp;
procedure ParseDeclarations;
procedure SkipBlock( termid1, termid2 : identifier := keyword_t );
procedure ParseBlock( termid1, termid2 : identifier := keyword_t );
procedure ParseEndOfStatement;
procedure ParseCompletedStatement;
procedure ParseShellCommand;
procedure ParseGeneralStatement;

---------------------------------------------------------
-- HOUSEKEEPING and SETUP
---------------------------------------------------------

procedure interpret;
procedure interpretScript( scriptPath : string );

procedure startParser;
procedure shutdownParser;
procedure Parse;

end parser;
