------------------------------------------------------------------------------
-- BUSH Parser Aux (Parser Support)                                         --
--                                                                          --
-- Part of BUSH                                                             --
------------------------------------------------------------------------------
--                                                                          --
--                 Copyright (C) 2001 Ken O. Burtch & FSF                   --
--                                                                          --
-- This is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion.  This is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with this;  see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- This is maintained at http://www.vaxxine.com/pegasoft                    --
--                                                                          --
------------------------------------------------------------------------------

--with system, ada.strings.unbounded
--    bush_os, user_io, script_io, scanner, builtins, run_cmd, string_util;
--use ada.strings.unbounded,
--    bush_os, user_io, script_io, scanner, builtins, run_cmd, string_util;

with ada.text_io, Interfaces.C;
use  ada.text_io, Interfaces.C;
--with Ada.Text_IO, Interfaces.C, System.Address_To_Access_Conversions;
--use Ada.Text_IO, Interfaces.C;

with string_util;
use  string_util;

package body parser_aux is

use bush_os.HEptrs;

function isExecutingCommand return boolean is
  -- True if OK to execute a statement that does work.
  -- That is, the parser isn't skipping the line because of
  -- an error or exiting a block.
begin
  return not error_found and not exit_block and not syntax_check;
end isExecutingCommand;
--pragma inline( isExecutingCommand );

procedure discardUnusedIdentifier( id : identifier ) is
  -- if an identifier has been not been assigned a type,
  -- assume it's unused and discard it.
  b : boolean;
begin
  if id /= eof_t then
     if identifiers( id ).kind = new_t or identifiers( id ).kind = eof_t then
        b := deleteIdent( id );
     end if;
  end if;
end discardUnusedIdentifier;

procedure makeTempFile( s : out unbounded_string ) is
  -- create a unique temporary filename
  LinuxPath : string := "/tmp/bushXXXXXX" & ASCII.NUL;
begin
  s := null_unbounded_string;
  mktemp( LinuxPath );
  for i in aLinuxPath'range loop
      exit when LinuxPath( i ) = ASCII.NUL;
      s := s & LinuxPath( i );
  end loop;
end makeTempFile;

--function isEOF( fd : aFileDescriptor ) return boolean is
-- true if the file descriptor is at the end of file
--  ch     : character;
--  result : long_integer;
--  seekResult : long_integer;
--begin
--  read( result, fd, ch, 1 );                 -- read a single character
--  if result > 0 then                         -- read something?
--     seekResult := lseek( fd, -1, WHENCE_SEEK_CUR); -- move back one character
--  end if;
--  return result < 1;                         -- return if read reported EOF
--end isEOF;

function stringField( i : identifier; f : natural ) return unbounded_string is
-- same as string_util.stringField except users an identifier's value
begin
  return stringField( identifiers( i ).value, ASCII.NUL, f );
end stringField;

procedure replaceField( i : identifier; f : natural; field : string ) is
-- same as string_util.replaceField except users an identifier's value
begin
  replaceField( identifiers( i ).value, ASCII.NUL, f, field );
end replaceField;

function OSerror( e : integer ) return string is
-- return an OS error message for error number e
  lastchar : natural := 0;
  ep       : anErrorPtr;
begin
  ep := strerror( e );
  for i in ep.all'range loop
      if ep(i) = ASCII.NUL then
	 lastchar := i-1;
         exit;
      end if;
  end loop;
  return string( ep( 1..lastchar ) );
end OSerror;

function openSocket( serverName : unbounded_string; port : integer ) return aSocketFD is

--  procedure SendHTTPCommand( soc : aSocketFD; cmd : string ) is
   -- Write a HTTP command string to the socket
--    amountWritten : integer := 0;
--    totalWritten  : integer := 0;
--    position      : integer := cmd'first;
--  begin
--loop
--Write( amountWritten, soc, cmd( position )'address,
--            cmd'length - integer( totalWritten ) );
--          if amountWritten < 0 then
--Put_Line( Standard_Error, "Write to socket failed" );
--             return;
--          end if;
--          Put_Line( "Sent" & amountWritten'img & " bytes" );
--          totalWritten := totalWritten + amountWritten;
--          position := position + amountWritten;
--    exit when totalWritten = cmd'length;
--    end loop;
--  end SendHTTPCommand;

--  ServerName: string := "hyde";

  mySocket    : aSocketFD;     -- the socket
  myAddress   : aSocketAddr;   -- where it goes
  myServer    : aHEptr;        -- IP number of server
  myServerPtr : HEptrs.Object_Pointer;
  addrList    : addrListPtrs.Object_Pointer;
  Result      : int;

begin

  -- initialize a new TCP/IP socket
  -- 0 for the third param lets the kernel decide

  --Put_Line( "Initializing a TCP/IP socket" );
  --Put_Line( "Socket( " & PF_INET'img & ',' & SOCK_STREAM'img & ", 0 );" );

  mySocket := Socket( PF_INET, SOCK_STREAM, 0 );
  if mySocket = -1 then
     err( "error making socket: " & OSError( errno ) );
     return -1;
  end if;
  --New_Line;

  -- Lookup the IP number for the server

  --Put_Line( "Looking for information on " & to_string( serverName ) );

  myServer := GetHostByName( to_string( serverName ) & ASCII.NUL );
  myServerPtr := HEptrs.To_Pointer( myServer );
  if myServerPtr = null then
     if errno = 0 then
        err( "there is no server by this name" );
     else
        err( "error looking up host: " & OSError( errno ) );
     end if;
     return -1;
  end if;

  --Put_Line( "IP number is" & myServerPtr.h_length'img & " bytes long" );
  addrList := addrlistPtrs.To_Pointer( myServerPtr.h_addr_list );
  --New_Line;

  -- Create the IP, port and protocol information

  --Put_Line( "Preparing connection destination information" );
  myAddress.family := AF_INET;
  myAddress.port   := htons( Interfaces.C.Unsigned_Short( port ) );
  memcpy( myAddress.ip'address, addrlist.all, myServerPtr.h_length );
  --New_Line;

  -- Open a connection to the server

  --Put_Line( "Connect( Result, Socket, Family/Address rec, F/A rec size )" );

  Connect( Result, mySocket, myAddress, myAddress'size/8 );

  --Put( "Connect( " & Result'img & "," );
  --Put(  myAddress.family'img & "/" );
  --PutIPNum( myAddress.ip );
  --Put(  "," & integer'image( myAddress'size / 8 ) & ")" );
  if Result /= 0 then
     err( "error connecting to server: " & OSerror( errno ) );
     return -1;
  end if;
  --New_Line;

  return mySocket;

end openSocket;

end parser_aux;

