------------------------------------------------------------------------------
-- BUSH Strings Package Parser                                              --
--                                                                          --
-- Part of BUSH                                                             --
------------------------------------------------------------------------------
--                                                                          --
--                 Copyright (C) 2001 Ken O. Burtch & FSF                   --
--                                                                          --
-- This is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion.  This is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with this;  see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- This is maintained at http://www.vaxxine.com/pegasoft                    --
--                                                                          --
------------------------------------------------------------------------------

with gnat.regexp,
    gnat.regpat,
    scanner,
    string_util,
    parser_aux,
    parser,
    bush_os;
use gnat.regexp,
    gnat.regpat,
    scanner,
    string_util,
    parser_aux,
    parser,
    bush_os;

package body parser_strings is


procedure ParseStringsGlob( result : out unbounded_string ) is
  -- Syntax: glob( expr, string )
  -- Source: GNAT.RegExp.Match
  expr_val  : unbounded_string;
  expr_type : identifier;
  pat_val   : unbounded_string;
  pat_type  : identifier;
  re        : regexp;
  b         : boolean;
begin
  result := null_unbounded_string;
  expect( glob_t );
  expect( symbol_t, "(" );
  ParseExpression( pat_val, pat_type );
  if uni_types_ok( pat_type, uni_string_t ) then
     expect( symbol_t, "," );
     ParseExpression( expr_val, expr_type );
     if uni_types_ok( expr_type, uni_string_t ) then
        expect( symbol_t, ")" );
        if isExecutingCommand then
           begin
	     re := compile( to_string( pat_val ), glob => true,
	        case_sensitive => true );
	     b := match( to_string( expr_val ), re );
           exception when expression_error =>
	     err( "bad globbing expression '" & to_string( pat_val ) & "'" );
	     b := false;
           when storage_error =>
	     err( "formula too complex (storage_error exception)" );
	     b := false;
           when others =>
	     err( "exception raised in gnat.regexp.match" );
	     b := false;
	   end;
	   if not error_found then
	      if b then
	         result := to_unbounded_string( "1" );
              else
	         result := to_unbounded_string( "0" );
              end if;
	   end if;
        end if;
     end if;
  end if;
end ParseStringsGlob;

procedure ParseStringsMatch( result : out unbounded_string ) is
  -- Syntax: match( file )
  -- Source: GNAT.RegPat.Match
  expr_val  : unbounded_string;
  expr_type : identifier;
  pat_val   : unbounded_string;
  pat_type  : identifier;
  b         : boolean;
begin
  result := null_unbounded_string;
  get_next_token;
  expect( symbol_t, "(" );
  ParseExpression( pat_val, pat_type );
  if uni_types_ok( pat_type, uni_string_t ) then
     expect( symbol_t, "," );
     ParseExpression( expr_val, expr_type );
     if uni_types_ok( expr_type, uni_string_t ) then
        expect( symbol_t, ")" );
        if isExecutingCommand then
	   begin
	     b := match( to_string( pat_val ), to_string( expr_val ) );
           exception when expression_error =>
	     err( "bad regular expression '" & to_string( pat_val ) & "'" );
	     b := false;
           when storage_error =>
	     err( "formula too complex (storage_error exception)" );
	     b := false;
           when program_error =>
	     err( "program_error exception raised gnat.regpat.match" );
	     b := false;
           when others =>
	     err( "exception raised in gnat.regpat.match" );
	     b := false;
	   end;
	   if not error_found then
	      if b then
	         result := to_unbounded_string( "1" );
              else
	         result := to_unbounded_string( "0" );
              end if;
	      if trace then
	         if b then
		   put_trace( "'" & to_string( pat_val ) &
		   "' pattern matches string '" &
		   to_string( expr_val ) &
		   "'" );
                 else
	           put_trace( "'" & to_string( pat_val ) &
	  	   "' pattern does not match string '" &
		   to_string( expr_val ) &
		   "'" );
                 end if;
              end if;
	   end if;
        end if;
     end if;
  end if;
end ParseStringsMatch;

procedure ParseStringsElement( result : in out unbounded_string ) is
  -- Syntax: element( s, i )
  -- Source: Ada.Strings.Unbounded.Element
  str_val : unbounded_string;
  str_type : identifier;
  index_val : unbounded_string;
  index_type : identifier;
begin
  expect( element_t );
  expect( symbol_t, "(" );
  ParseExpression( str_val, str_type );
  if uni_types_ok( str_type, string_t ) then
     expect( symbol_t, "," );
     ParseExpression( index_val, index_type );
     if base_types_ok( index_type, positive_t ) then
        expect( symbol_t, ")" );
     end if;
  end if;
  begin
     if isExecutingCommand then
        result := to_unbounded_string( "" &
           Element( str_val, positive'value( to_string( index_val ) ) ) );
     end if;
  exception when others =>
     err( "exception raised" );
  end;
end ParseStringsElement;

procedure ParseStringsSlice( result : in out unbounded_string ) is
  -- Syntax: slice( s, l, h )
  -- Source: Ada.Strings.Unbounded.Slice
  str_val  : unbounded_string;
  str_type : identifier;
  low_val  : unbounded_string;
  low_type : identifier;
  hi_val   : unbounded_string;
  hi_type  : identifier;
begin
  expect( slice_t );
  expect( symbol_t, "(" );
  ParseExpression( str_val, str_type );
  if uni_types_ok( str_type, string_t ) then
     expect( symbol_t, "," );
     ParseExpression( low_val, low_type );
     if base_types_ok( low_type, positive_t ) then
        expect( symbol_t, "," );
        ParseExpression( hi_val, hi_type );
        if base_types_ok( hi_type, natural_t ) then
           expect( symbol_t, ")" );
        end if;
     end if;
  end if;
  begin
     if isExecutingCommand then
        result := to_unbounded_string(
           Slice( str_val,
	     positive'value( to_string( low_val ) ),
	     natural'value( to_string( hi_val ) )
	   ) );
     end if;
  exception when others =>
     err( "exception raised" );
  end;
end ParseStringsSlice;

procedure ParseStringsIndex( result : in out unbounded_string ) is
  -- Syntax: strings.index( s, p )
  -- Source: Ada.Strings.Unbounded.Index
  str_val : unbounded_string;
  str_type : identifier;
  pat_val : unbounded_string;
  pat_type : identifier;
begin
  expect( index_t );
  expect( symbol_t, "(" );
  ParseExpression( str_val, str_type );
  if uni_types_ok( str_type, string_t ) then
     expect( symbol_t, "," );
     ParseExpression( pat_val, pat_type );
     if base_types_ok( pat_type, string_t ) then
        expect( symbol_t, ")" );
     end if;
  end if;
  begin
     if isExecutingCommand then
        result := to_unbounded_string( Index( str_val, to_string( pat_val ) )'img );
     end if;
  exception when others =>
     err( "exception raised" );
  end;
end ParseStringsIndex;

procedure ParseStringsIndexNonBlank( result : in out unbounded_string ) is
  -- Syntax: strings.index_non_blank( s )
  -- Source: Ada.Strings.Unbounded.Index_Non_Blank
  str_val : unbounded_string;
  str_type : identifier;
begin
  expect( index_non_blank_t );
  expect( symbol_t, "(" );
  ParseExpression( str_val, str_type );
  if uni_types_ok( str_type, string_t ) then
     expect( symbol_t, ")" );
  end if;
  begin
     if isExecutingCommand then
        result := to_unbounded_string( Index_Non_Blank( str_val )'img );
     end if;
  exception when others =>
     err( "exception raised" );
  end;
end ParseStringsIndexNonBlank;

procedure ParseStringsCount( result : in out unbounded_string ) is
  -- Syntax: strings.count( s, p )
  -- Source: Ada.Strings.Unbounded.Count
  str_val : unbounded_string;
  str_type : identifier;
  pat_val : unbounded_string;
  pat_type : identifier;
begin
  expect( count_t );
  expect( symbol_t, "(" );
  ParseExpression( str_val, str_type );
  if uni_types_ok( str_type, string_t ) then
     expect( symbol_t, "," );
     ParseExpression( pat_val, pat_type );
     if base_types_ok( pat_type, string_t ) then
        expect( symbol_t, ")" );
     end if;
  end if;
  begin
     if isExecutingCommand then
        result := to_unbounded_string( Ada.Strings.Unbounded.Count( str_val,
           to_string( pat_val ) )'img );
     end if;
  exception when others =>
     err( "exception raised" );
  end;
end ParseStringsCount;

procedure ParseStringsReplaceSlice( result : in out unbounded_string ) is
  -- Syntax: strings.replace_slice( s, l, h, b )
  -- Source: Ada.Strings.Unbounded.Replace_Slice
  str_val : unbounded_string;
  str_type : identifier;
  low_val : unbounded_string;
  low_type : identifier;
  hi_val : unbounded_string;
  hi_type : identifier;
  by_val : unbounded_string;
  by_type : identifier;
begin
  expect( replace_slice_t );
  expect( symbol_t, "(" );
  ParseExpression( str_val, str_type );
  if uni_types_ok( str_type, string_t ) then
     expect( symbol_t, "," );
     ParseExpression( low_val, low_type );
     if base_types_ok( low_type, positive_t ) then
        expect( symbol_t, "," );
        ParseExpression( hi_val, hi_type );
        if base_types_ok( hi_type, natural_t ) then
           expect( symbol_t, "," );
           ParseExpression( by_val, by_type );
           if base_types_ok( by_type, string_t ) then
              expect( symbol_t, ")" );
           end if;
        end if;
     end if;
  end if;
  begin
     if isExecutingCommand then
        result := Replace_Slice( str_val,
	   positive'value( to_string( low_val ) ),
	   natural'value( to_string( hi_val ) ),
	   to_string( by_val )
	);
     end if;
  exception when others =>
     err( "exception raised" );
  end;
end ParseStringsReplaceSlice;

procedure ParseStringsInsert( result : in out unbounded_string ) is
  -- Syntax: strings.insert( s, b, n )
  -- Source: Ada.Strings.Unbounded.Replace_Slice
  str_val : unbounded_string;
  str_type : identifier;
  before_val : unbounded_string;
  before_type : identifier;
  new_val : unbounded_string;
  new_type : identifier;
begin
  expect( insert_t );
  expect( symbol_t, "(" );
  ParseExpression( str_val, str_type );
  if uni_types_ok( str_type, string_t ) then
     expect( symbol_t, "," );
     ParseExpression( before_val, before_type );
     if base_types_ok( before_type, positive_t ) then
        expect( symbol_t, "," );
        ParseExpression( new_val, new_type );
        if base_types_ok( new_type, string_t ) then
           expect( symbol_t, ")" );
        end if;
     end if;
  end if;
  begin
     if isExecutingCommand then
        result := Insert( str_val,
	   positive'value( to_string( before_val ) ),
	   to_string( new_val )
	);
     end if;
  exception when others =>
     err( "exception raised" );
  end;
end ParseStringsInsert;

procedure ParseStringsOverwrite( result : in out unbounded_string ) is
  -- Syntax: strings.overwrite( s, p, n )
  -- Source: Ada.Strings.Unbounded.Overwrite
  str_val  : unbounded_string;
  str_type : identifier;
  pos_val  : unbounded_string;
  pos_type : identifier;
  new_val  : unbounded_string;
  new_type : identifier;
begin
  expect( overwrite_t );
  expect( symbol_t, "(" );
  ParseExpression( str_val, str_type );
  if uni_types_ok( str_type, string_t ) then
     expect( symbol_t, "," );
     ParseExpression( pos_val, pos_type );
     if base_types_ok( pos_type, positive_t ) then
        expect( symbol_t, "," );
        ParseExpression( new_val, new_type );
        if base_types_ok( new_type, string_t ) then
           expect( symbol_t, ")" );
        end if;
     end if;
  end if;
  begin
     if isExecutingCommand then
        result := Overwrite( str_val,
	   positive'value( to_string( pos_val ) ),
	   to_string( new_val )
	);
     end if;
  exception when others =>
     err( "exception raised" );
  end;
end ParseStringsOverwrite;

procedure ParseStringsDelete( result : in out unbounded_string ) is
  -- Syntax: strings.delete( s, l, h )
  -- Source: Ada.Strings.Unbounded.Delete
  str_val  : unbounded_string;
  str_type : identifier;
  low_val  : unbounded_string;
  low_type : identifier;
  hi_val   : unbounded_string;
  hi_type  : identifier;
begin
  expect( sdelete_t );
  expect( symbol_t, "(" );
  ParseExpression( str_val, str_type );
  if uni_types_ok( str_type, string_t ) then
     expect( symbol_t, "," );
     ParseExpression( low_val, low_type );
     if base_types_ok( low_type, positive_t ) then
        expect( symbol_t, "," );
        ParseExpression( hi_val, hi_type );
        if base_types_ok( hi_type, natural_t ) then
           expect( symbol_t, ")" );
        end if;
     end if;
  end if;
  begin
     if isExecutingCommand then
        result := Delete( str_val,
	   positive'value( to_string( low_val ) ),
	   natural'value( to_string( hi_val ) )
	);
     end if;
  exception when others =>
     err( "exception raised" );
  end;
end ParseStringsDelete;

procedure ParseStringsTrim( result : in out unbounded_string ) is
  -- Syntax: strings.trim( s )
  -- Source: Ada.Strings.Unbounded.Trim
  str_val : unbounded_string;
  str_type : identifier;
begin
  expect( trim_t );
  expect( symbol_t, "(" );
  ParseExpression( str_val, str_type );
  if uni_types_ok( str_type, string_t ) then
     expect( symbol_t, ")" );
  end if;
  begin
     if isExecutingCommand then
        result := trim( str_val, Ada.Strings.Both );
     end if;
  exception when others =>
     err( "exception raised" );
  end;
end ParseStringsTrim;

procedure ParseStringsLength( result : in out unbounded_string ) is
  -- Syntax: strings.trim( s )
  -- Source: Ada.Strings.Unbounded.Trim
  str_val : unbounded_string;
  str_type : identifier;
begin
  expect( length_t );
  expect( symbol_t, "(" );
  ParseExpression( str_val, str_type );
  if uni_types_ok( str_type, string_t ) then
     expect( symbol_t, ")" );
  end if;
  begin
     if isExecutingCommand then
        result := to_unbounded_string( length( str_val )'img );
     end if;
  exception when others =>
     err( "exception raised" );
  end;
end ParseStringsLength;

procedure ParseStringsHead( result : in out unbounded_string ) is
  -- Syntax: strings.head( s, c )
  -- Source: Ada.Strings.Unbounded.Head
  str_val  : unbounded_string;
  str_type : identifier;
  cnt_val  : unbounded_string;
  cnt_type : identifier;
begin
  expect( head_t );
  expect( symbol_t, "(" );
  ParseExpression( str_val, str_type );
  if uni_types_ok( str_type, string_t ) then
     expect( symbol_t, "," );
     ParseExpression( cnt_val, cnt_type );
     if uni_types_ok( cnt_type, natural_t ) then
        expect( symbol_t, ")" );
     end if;
  end if;
  begin
     if isExecutingCommand then
        result := head( str_val, natural'value( to_string( cnt_val ) ) );
     end if;
  exception when others =>
     err( "exception raised" );
  end;
end ParseStringsHead;

procedure ParseStringsTail( result : in out unbounded_string ) is
  -- Syntax: strings.tail( s, c )
  -- Source: Ada.Strings.Unbounded.Tail
  str_val  : unbounded_string;
  str_type : identifier;
  cnt_val  : unbounded_string;
  cnt_type : identifier;
begin
  expect( tail_t );
  expect( symbol_t, "(" );
  ParseExpression( str_val, str_type );
  if uni_types_ok( str_type, string_t ) then
     expect( symbol_t, "," );
     ParseExpression( cnt_val, cnt_type );
     if uni_types_ok( cnt_type, natural_t ) then
        expect( symbol_t, ")" );
     end if;
  end if;
  begin
     if isExecutingCommand then
        result := tail( str_val, natural'value( to_string( cnt_val ) ) );
     end if;
  exception when others =>
     err( "exception raised" );
  end;
end ParseStringsTail;

procedure ParseStringsField( result : in out unbounded_string ) is
  -- Syntax: strings.field( s, c, d )
  -- Source: Ada.Strings.Unbounded.Tail
  str_val  : unbounded_string;
  str_type : identifier;
  cnt_val  : unbounded_string;
  cnt_type : identifier;
  del_val  : unbounded_string;
  del_type : identifier;
begin
  expect( field_t );
  expect( symbol_t, "(" );
  ParseExpression( str_val, str_type );
  if uni_types_ok( str_type, string_t ) then
     expect( symbol_t, "," );
     ParseExpression( del_val, del_type );
     if base_types_ok( del_type, character_t ) then
        expect( symbol_t, "," );
        ParseExpression( cnt_val, cnt_type );
        if uni_types_ok( cnt_type, natural_t ) then
           expect( symbol_t, ")" );
        end if;
     end if;
  end if;
  begin
     if isExecutingCommand then
        result := stringField( str_val, Element( del_val, 1 ), natural'value( to_string( cnt_val ) ) );
     end if;
  exception when others =>
     err( "exception raised" );
  end;
end ParseStringsField;

procedure ParseStringsMkTemp( result : in out unbounded_string ) is
  -- Syntax: strings.mktemp
  -- Source: BUSH builtin
  str_val : unbounded_string;
  str_type : identifier;
begin
  expect( mktemp_t );
  expect( symbol_t, "(" );
  ParseExpression( str_val, str_type );
  if uni_types_ok( str_type, string_t ) then
     expect( symbol_t, ")" );
  end if;
  if isExecutingCommand then
     declare
       LinuxPath : string := to_string( str_val ) & "XXXXXX" & ASCII.NUL;
     begin
       result := null_unbounded_string;
       mktemp( LinuxPath );
       for i in aLinuxPath'range loop
         exit when LinuxPath( i ) = ASCII.NUL;
         result := result & LinuxPath( i );
       end loop;
     end;
  end if;
end ParseStringsMkTemp;

procedure ParseStringsVal( result : in out unbounded_string ) is
  -- Syntax: strings.val( natural );
  -- Source: Ada 'val attribute
  expr_val   : unbounded_string;
  expr_type  : identifier;
begin
  expect( val_t );
  expect( symbol_t, "(" );
  ParseExpression( expr_val, expr_type );
  if base_types_ok( expr_type, natural_t ) then
     expect( symbol_t, ")" );
  end if;
  begin
    if isExecutingCommand then
       result := to_unbounded_string( "" & character'val( natural'value( to_string( expr_val ) ) ) );
    end if;
  exception when others =>
    err( "exception raised" );
  end;
end ParseStringsVal;

procedure ParseStringsImage( result : in out unbounded_string ) is
  -- Syntax: strings.image( x );
  -- Source: Ada 'image attribute
  expr_val   : unbounded_string;
  expr_type  : identifier;
begin
  expect( image_t );
  expect( symbol_t, "(" );
  ParseExpression( expr_val, expr_type );
  if uni_types_ok( expr_type, uni_numeric_t ) then
     expect( symbol_t, ")" );
  end if;
  begin
    if isExecutingCommand then
       result := expr_val;
    end if;
  exception when others =>
    err( "exception raised" );
  end;
end ParseStringsImage;

end parser_strings;
