
------------------------------------------------------------------------------
-- Reading the keyboard, writing to the terminal/console                    --
--                                                                          --
-- Part of BUSH                                                             --
------------------------------------------------------------------------------
--                                                                          --
--                 Copyright (C) 2001 Ken O. Burtch & FSF                   --
--                                                                          --
-- This is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion.  This is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with this;  see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- This is maintained at http://www.vaxxine.com/pegasoft                    --
--                                                                          --
------------------------------------------------------------------------------

with ada.text_io, ada.strings.unbounded.text_io;
use  ada.text_io, ada.strings.unbounded.text_io;


package body user_io is


function getTerminalInfo( ti : string ) return unbounded_string is
-- read a termcap value using tput.  Really kludgy using
-- system(). tgetnam() requires curses.
  tempfile : unbounded_string;
  result   : unbounded_string;
  tputFile : aFileDescriptor;
  amountRead : long_integer;
  status   : integer;
  ch       : character;
begin
  tempFile := to_unbounded_string( getpid'img );
  delete( tempFile, 1, 1 );
  tempFile := to_unbounded_string( "/tmp/bush_tput." ) & tempfile;
  status := linux_system( "tput " & ti & "> " & to_string( tempfile ) &
    ASCII.NUL );
  if status /= 0 then
     put_line( standard_error, "unable to get terminal info" );
     return null_unbounded_string;
  end if;
  tputFile := open( to_string( tempfile ) & ASCII.NUL, 0, 660 );
  result := null_unbounded_string;
  loop
    read( amountRead, tputFile, ch, 1 );
    exit when amountRead <= 0;
    result := result & ch;
  end loop;
  close( tputFile );
  if unlink( to_string( tempfile ) & ASCII.NUL ) /= 0 then
     put_line( standard_error, "unable to delete temp file " & tempfile );
  end if;
  return result;
end getTerminalInfo;

procedure getDisplayInfo( sz : in out winsz_info ) is
-- determine the dimensions of the screen using ioctl()
-- (and if that fails, try tput)
  res : integer;
  ttyFile : aFileDescriptor;
begin
  sz.row := 0; -- defaults
  sz.col := 0;
  ttyFile := open( "/dev/tty" & ASCII.NUL, 0, 660 );
  if ttyFile >= 0 then
     ioctl_TIOCGWINSZ( res, ttyFile, TIOCGWINSZ, sz );
     if res < 0 or sz.row <= 0 then
       sz.row := short_integer'value( to_string( getTerminalInfo( "lines" ) ) );
     end if;
     if res < 0 or sz.col <= 1 then
       sz.col := short_integer'value( to_string( getTerminalInfo( "cols" ) ) );
     end if;
     if sz.row <= 0 or sz.col <= 1 then
        put_line( standard_error, "unable to determine display info" );
        sz.row := 24;
        sz.col := 80;
     end if;
     close( ttyFile );
  else
     put_line( standard_error, "unable to open /dev/tty" );
  end if;
  term_normal := getTerminalInfo( "sgr0" );
  term_bold   := getTerminalInfo( "bold" );
  term_cleop  := getTerminalInfo( "ed" );
  term_cleol  := getTerminalInfo( "el" );
  term_up     := getTerminalInfo( "cuu1" );
  term_right  := getTerminalInfo( "cuf1" );
  term_reset  := getTerminalInfo( "reset" );
  term_clear  := getTerminalInfo( "clear" );
  exception when others =>
    put_line( standard_error, "exception thrown while determining display info"
);
    sz.row := 24;
    sz.col := 80;
end getDisplayInfo;

procedure beep is
-- beep the terminal
begin
  put( standard_error, ASCII.BEL );
  -- (should really be flushed)
end beep;

procedure getKey( ch : out character ) is
-- read a single keypress and don't show the character to user
  tio     : termios;         -- tty setting for raw input
  oldtio  : termios;         -- previous tty settings
  amountRead : long_integer; -- characters read
  ttyFile : aFileDescriptor; -- our controlling tty, for ioctl
  res     : integer;         -- results
begin
  --pragma assert( tio'size = 60*8 ); -- for HP-UX
  if not isatty( stdin ) then                      -- not a tty?
     read( amountRead, stdin, ch, 1 );             -- read a character
     if amountRead = 0 then                        -- nothing read?
        ch := ASCII.EOT;                           -- return a control-d
     end if;
  else                                         
     -- read character here (non-canonical, set by termios
     -- MIN = 1  & TIME = 0).  Disable special chars separately
     -- strip high bit.  Normally, canonical + 0 min + 0 time
     ttyFile := open( "/dev/tty" & ASCII.NUL, 0, 660 );
     if ttyFile < 0 then
        put_line( "open failed " & errno'img );
     else
        ioctl_getattr( res, ttyFile, TCGETATTR, tio );
        if res /= 0 then
           put_line( "ioctl get attr failed" );
        else
           oldtio := tio;
--put_line( "iflag = " & tio.c_iflag'img );
--put_line( "lflag = " & tio.c_lflag'img );
--put_line( "cc_intr = " & character'pos( tio.cc_intr )'img );
--put_line( "cc_quit = " & character'pos( tio.cc_quit )'img );

--put_line( "cc_erase= " & character'pos( tio.cc_erase )'img );
--put_line( "cc_kill = " & character'pos( tio.cc_kill )'img );
--put_line( "cc_eof  = " & character'pos( tio.cc_eof  )'img );
--put_line( "cc_time = " & character'pos( tio.cc_time )'img );
--put_line( "cc_min  = " & character'pos( tio.cc_min  )'img );
--put_line( "cc_swtc = " & character'pos( tio.cc_swtc )'img );
--put_line( "cc_star = " & character'pos( tio.cc_start )'img );
--put_line( "cc_stop = " & character'pos( tio.cc_stop )'img );
--put_line( "cc_eol  = " & character'pos( tio.cc_eol )'img );
           -- these are based on "Linug Application Development"
           tio.c_lflag := tio.c_lflag and not ICANON;  -- no special chars
           tio.c_lflag := tio.c_lflag and not ECHO;    -- no print
           tio.c_lflag := tio.c_lflag and not ECHONL;  -- ditto
           tio.c_lflag := tio.c_lflag and not ECHOCTL; -- ditto
           tio.cc_min := character'val(1);   -- 1 character
           tio.cc_time := character'val(0);  -- no timeout
           -- these are based on Bash readline library, rltty.c
           -- this assumes no software flow control
           tio.c_iflag := tio.c_iflag and not IXON;    -- ctrl-s through
           tio.c_iflag := tio.c_iflag and not IXOFF;   -- ctrl-q through
           tio.c_iflag := tio.c_iflag and not IXANY;   -- shouldn't be needed
           tio.c_iflag := tio.c_iflag and not ICRNL;   -- no CR->NL conversion
           tio.c_iflag := tio.c_iflag and not INLCR;   -- or vice versa
           -- ISIG and some other stuff done by readline not
           -- done here.  Use ISIG to allow ctrl-c through.  For testing,
           -- leave alone.
           ioctl_setattr( res, ttyFile, TCSETATTR, tio ); -- raw read mode
           if res /= 0 then                            -- very unlikely
              put_line( "ioctl_set failed" );          -- but check anyway
              -- probably should raise an exception here
           else
              res := tcdrain( stdout );            -- flush pending output
              read( amountRead, stdin, ch, 1 );    -- read a character
              if amountRead = 0 then               -- nothing read?
                 ch := ASCII.EOT;                  -- return a control-d
              end if;
              ioctl_setattr( res, ttyFile, TCSETATTR, oldtio ); -- restore settings
           end if;
        end if;
        close( ttyfile );
     end if;
     if ch > ASCII.DEL then
        ch := character'val( character'pos( ch ) - 128 );
     end if;
  end if;
end getKey;

procedure getLine( line : out unbounded_string; keepHistory : boolean := false ) is
-- read a line from the keyboard
-- Note: the keymap is hard-coded, but really should be taken from
-- current terminal settings.

  procedure redrawLine( oldPos : natural; newPos : natural := 0 ) is
    oldLines  : integer; -- number of lines of old text
    oldChars  : integer; -- number of characters from margin
  begin
    -- these two formulas are not quite right
    if keepHistory then
       oldLines := (prompt'length + oldPos ) / integer( displayInfo.col );
       oldChars  := (prompt'length + oldPos) -
          ( oldLines * integer( displayInfo.col ) );
    else
       oldLines := oldPos / integer( displayInfo.col );
       oldChars  := oldPos - ( oldLines * integer( displayInfo.col ) );
    end if;
    -- position at start of old line
    for i in 1..oldChars-1 loop
        put( ASCII.BS );
    end loop;
    for i in 1..oldLines loop
        put( term_up );
    end loop;
    -- draw new line
    if keepHistory then
       put( term_bold );
       put( prompt );
       put( term_normal );
    end if;
    put( line );
    put( term_cleol );
    -- reposition cursor, if desired
    if newPos > 0 then
       for i in newPos..length( line ) loop   -- reposition cursor
           put( ASCII.BS );                   -- (not very efficient)
       end loop;
    end if;
    --put( " - " );
    --put( oldchars'img );
    --put( displayInfo.col'img );
  end redrawLine;

  vi_escape  : boolean := false; -- true if expecting vi-mode command
  arrow_escape : boolean := false; -- true if expecting VT-100 arrow
  ch         : character; -- last character read
  pos        : natural;   -- position to insert next character
  old_pos    : natural;   -- for redrawing lines
  histpos    : integer;   -- position in history
  crit_pos   : natural;   -- for searching history
  criteria   : unbounded_string;

begin
  line     := null_unbounded_string;
  criteria := null_unbounded_string;
  pos      := 1;
  histpos  := historyNext;

  loop
<<retry>> getKey( ch );

    -- VI ESCAPE MODE HANDLING
    --
    -- process sequence and convert to its emacs mode equivalent

    if ch = ASCII.ESC then
       vi_escape := not vi_escape;            -- enter/exit vi escape mode
       goto retry;                            -- and go get next key
    elsif arrow_escape then                   -- ESC + [ + ...?
       arrow_escape := false;                 -- got our arrow code
       if ch = 'A' then                       -- A = up
          ch := ASCII.DLE;
       elsif ch = 'B' then                    -- B = down
          ch := ASCII.SO;
       elsif ch = 'C' then                    -- C = right
          ch := ASCII.ACK;
       elsif ch = 'D' then                    -- D = left
          ch := ASCII.STX;
       else
          beep;                               -- otherwise unknown
       end if;
    elsif vi_escape then
       if ch ='[' then                        -- same as control-n
	  arrow_escape := true;
          vi_escape := false;                 -- cancel vi mode
	  goto retry;
       elsif ch ='j' then                     -- same as control-n
          ch := ASCII.SO;
       elsif ch = 'k' then                    -- same as control-p
          ch := ASCII.DLE;
       elsif ch = 'h' then                    -- same as control-b
          ch := ASCII.STX;
       elsif ch = 'l' then                    -- same as control-f
          ch := ASCII.ACK;
       elsif ch = '^' then                    -- same as control-a
          ch := ASCII.SOH;
       elsif ch = '$' then                    -- same as control-e
          ch := ASCII.ENQ;
       else                                   -- else not supported
          vi_escape := false;                 -- drop out of vi escape mode
          beep;                               -- beep at illegal character
          goto retry;                         -- and get next key
       end if;
    elsif ch = ASCII.EOT then                 -- end of file, treat as end of input
       ch := ASCII.CR;
    end if;

    -- EMACS MODE HANDLING
          
    case ch is
    when ASCII.CR | ASCII.LF =>
       new_line;
       if length( line ) > 0 and keepHistory then -- don't save empty
          history( historyNext ) := line;     -- lines
          historyNext := historyNext + 1;
          if historyNext > historyArray'last then
             historyNext := 1;
          end if;
       end if;
       exit;
    when ASCII.BS | ASCII.DEL =>              -- control-h or delete key
       if pos > 1 then
          delete( line, pos-1, pos-1 );       -- delete char to left of cursor
          old_pos := pos;                     -- remember old position
          pos := pos - 1;                     -- new position is one to left
          redrawLine( old_pos, pos );         -- redraw line, cursor at end
       else                                   -- nothing to delete?

          put( ASCII.BEL );                   -- beep
       end if;
    when ASCII.STX =>                         -- control-b
       if pos > 1 then
          old_pos := pos;                     -- remember old position
          pos := pos - 1;                     -- new position is one to left
          put( ASCII.BS );                    -- move cursor
       else                                   -- no place to go?
          put( ASCII.BEL );                   -- beep
       end if;
    when ASCII.ACK =>                         -- control-f
          if pos <= length( line ) then
             old_pos := pos;                  -- remember old position
             pos := pos + 1;                  -- new position is one to right
             put( term_right );               -- move cursor
          else                                -- no place to go?
             put( ASCII.BEL );                -- beep
          end if;
    when ASCII.SO =>                          -- control-n
      if histpos = historyNext or not keepHistory then
         put( ASCII.BEL );
      else
         histpos := histpos + 1;
         if histpos > historyArray'last then
            histpos := 1;
         end if;
         old_pos := pos;                     -- remember old position
         line := history( histpos );
         pos := length( line ) + 1;          -- new position at end of string
         redrawLine( old_pos, pos );         -- redraw line, cursor at end
      end if;
    when ASCII.DLE =>                         -- control-p
      if not keepHistory then
	 put( ASCII.BEL );
      else
         old_pos := histpos;
         histpos := histpos - 1;
         if histpos < 1 then
            histpos := historyArray'last;
         end if;
         if histpos = historyNext then
            histpos := old_pos;
            put( ASCII.BEL );
         else
            old_pos := pos;                     -- remember old position
            line := history( histpos );
            pos := length( line ) + 1;          -- new position at end of string
            redrawLine( old_pos, pos );         -- redraw line, cursor at end
         end if;
      end if;
    when ASCII.CAN =>                     -- control-x
      old_pos := pos;                     -- remember old position
      pos := 1;                           -- empty string now
      line := null_unbounded_string;
      redrawLine( old_pos );              -- redraw line
    when ASCII.SOH =>                     -- control-a
      redrawLine( pos, 1 );
      pos := 1;                           -- fix position
    when ASCII.ENQ =>                     -- control-e
      old_pos := pos;                     -- remember old position
      pos := length( line ) + 1;          -- new position at end of string
      redrawLine( old_pos, pos );         -- redraw line, cursor at end
    when ASCII.DC2 =>                         -- control-r
      if pos = 1 then
         put( ASCII.BEL );
         goto retry;
      elsif pos > length( line ) then
         criteria := line;
      else
         criteria := Delete( line, pos, length( line ) );
      end if;
      old_pos := histpos;
      crit_pos := pos;
      loop
        histpos := histpos - 1;
        if histpos < 1 then
           histpos := historyArray'last;
        end if;
        if histpos = historyNext then         -- out of history?
           line := criteria;                  -- restore old line
           histpos := old_pos;                -- that's all folks
           put( ASCII.BEL );
           exit;
        end if;
        line := history( histpos );           -- next history line
        if Index( line, To_String( Criteria ) ) = 1 then -- matches criteria?
           pos := length( line ) + 1;         -- new position at end of string
           redrawLine( crit_pos, pos );       -- redraw line, cursor at end
           exit;
        end if;
      end loop;
    when ASCII.GS =>                          -- control-] (forward char search)
      old_pos := pos;
      getKey( ch );
      for i in pos+1..length( line ) loop
          if Element( line, i ) = ch then
             pos := i;
             exit;
          end if;
      end loop;
      if pos = old_pos then
         put( ASCII.BEL );
      else
         redrawLine( old_pos, pos );
      end if;
    when others =>
       if pos > length( line ) then           -- adding at end of line?
          line := line & ch;                  -- append character
          pos := pos + 1;                     -- new position is one to right
          put( ch );                          -- display character
       else
          insert( line, pos, ch & "" );       -- insert the character
          old_pos := pos;                     -- remember old position
          pos := pos + 1;                     -- new position is one to right
          redrawLine( old_pos, pos );         -- redraw line, cursor at end
       end if;
    end case;
  end loop;
end getLine;

end user_io;
