
------------------------------------------------------------------------------
-- Reading the keyboard, writing to the terminal/console                    --
--                                                                          --
-- Part of BUSH                                                             --
------------------------------------------------------------------------------
--                                                                          --
--                 Copyright (C) 2001 Ken O. Burtch & FSF                   --
--                                                                          --
-- This is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion.  This is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with this;  see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- This is maintained at http://www.vaxxine.com/pegasoft                    --
--                                                                          --
------------------------------------------------------------------------------

with system, ada.strings.unbounded, bush_os;
use ada.strings.unbounded, bush_os;


package user_io is


prompt  : constant string := "=> "; -- input prompt

displayInfo : winsz_info;
term_normal : unbounded_string; -- normal print
term_bold   : unbounded_string; -- bold
term_cleop  : unbounded_string; -- clear to end of page
term_cleol  : unbounded_string; -- clear to end of line
term_up     : unbounded_string; -- move cursor up
term_right  : unbounded_string; -- move cursor right
term_reset  : unbounded_string; -- reset terminal to defaults
term_clear  : unbounded_string; -- clear screen


------------------------------------------------------
-- Command History
--
-- Command history is a circular queue
------------------------------------------------------

type historyArray is array( 1..40 ) of unbounded_string;

history     : historyArray;     -- the previous commands
historyNext : integer;          -- next to be overwritten


------------------------------------------------------
-- Subprograms
--
------------------------------------------------------

function getTerminalInfo( ti : string ) return unbounded_string;
-- read a termcap value using tput.  Really kludgy using
-- system(). tgetnam() requires curses.

procedure getDisplayInfo( sz : in out winsz_info );
-- determine the dimensions of the screen using ioctl()
-- (and if that fails, try tput)

procedure beep;
-- beep the terminal

procedure getKey( ch : out character );
-- read a single keypress and don't show the character to user

procedure getLine( line : out unbounded_string; keepHistory : boolean := false );
-- read a line from the keyboard
-- Note: the keymap is hard-coded, but really should be taken from
-- current terminal settings.

end user_io;

